/*
*	AZN_TRunOSA.cp
*
*	A class using the OSA to run AppleScripts
*	from within an application
*
*	 Andrew Nemeth, Warrimoo Australia  1995
*	aznemeng@zeta.org.au
*
*	File Created:		24 Mar 95
*	File Ammended:		24, 26, 28 Mar;
*					35, 9, 27 Apr;
*					1, 2, 15 May;
*					18 Oct 95.
*/

#include	"AZN_TRunOSA.h"								//	load & run OSA AppleScripts

#include	"AZN_DBUG.H"									//	debugging utilites

#include	<OSAGeneric.h>									//	mac OSA tricks
#include	<Components.h>
#include	<OSA.h>
#include	<AppleScript.h>
#include	<AEObjects.h>									//	mac OSL constants
#include	<Gestalt.h>


//	CLASS STATIC VARIABLE
//
ComponentInstance	TRunOSA::scriptComponent = NULL;



TRunOSA::TRunOSA ()
	:	f_osaidOriginal( kOSANullScript ),
		f_osaidResult( kOSANullScript )
//
//	Put 'em up
//
{
	f_aedescScript.descriptorType = typeNull;
	f_aedescScript.dataHandle = NULL;
}



TRunOSA::~TRunOSA ()
//
//	Tear 'em down
//
{
	if ( NULL != TRunOSA::scriptComponent )
		{
		::AEDisposeDesc( &f_aedescScript );
		::OSADispose( TRunOSA::scriptComponent, f_osaidOriginal );
		::OSADispose( TRunOSA::scriptComponent, f_osaidResult );
		}
}



OSAError				TRunOSA::loadScript ( void )
//
//	Initialise and get ready for
//	script running
//
//	LOADS SCRIPT FROM APPLICATION's
//	RESOURCE FORK!
//
{
	Handle		hScript = NULL;
	OSAError		myOsaErr = noErr;

//														grab resource containing the script
	hScript = ::Get1IndResource( typeOSAGenericStorage, 1 );
	if ( NULL == hScript )
		return( resNotFound );
//														pry handle loose from resource map
	::DetachResource( hScript );
//														crank up class
	myOsaErr = myLoadScript( hScript );

	return( myOsaErr );
}



OSAError				TRunOSA::loadScript ( const FSSpec & fsspecScript )
//
//	Initialise and get ready for
//	script running
//
//	LOADS SCRIPT FROM FSSPEC PROVIDED!
//
{
	Handle		hScript = NULL;
	short		shResRefNum = -1;
	OSAError		myOsaErr = noErr;

//														open the file containing the compiled script
	shResRefNum = ::FSpOpenResFile( &fsspecScript, fsRdPerm );
	if ( -1 == shResRefNum || noErr != ::ResError() ) 
		return( resFNotFound );
//														grab resource containing the script
	hScript = ::Get1IndResource( typeOSAGenericStorage, 1 );
	if ( NULL == hScript )
		return( resNotFound );
//														pry handle loose & close file
	::DetachResource( hScript );
	::CloseResFile( shResRefNum );
//														crank up class
	myOsaErr = myLoadScript( hScript );

	return( myOsaErr );
}



OSAError				TRunOSA::myLoadScript ( Handle hScript )
//
//	Private method to load script 
//	and initialise class
//
//	DOES NOT DISPOSE OF HANDLE!
//	because handle becomes part of the
//	f_aedescScript created!
//
{
	const long		klgGestaltMask = 1L;
	long				lgFeature = 0L;
	AEDesc			aedescDummy = { typeNull, NULL };
	OSErr			myErr = noErr;
	OSAError			myOsaErr = noErr;

//														idiot filter
	ASSERT( NULL != hScript );
//														open a connection to the OSA (just once!)
	if ( NULL == TRunOSA::scriptComponent )
		{
//														gestalt to check for scripting support
		myErr = ::Gestalt( gestaltAppleEventsAttr, &lgFeature );
		if ( ( noErr == myErr ) && ( lgFeature & ( klgGestaltMask << gestaltScriptingSupport ) ) )
			NULL;
		else
			return( errOSACantOpenComponent );
//														load generic scripting component
		TRunOSA::scriptComponent = ::OpenDefaultComponent( kOSAComponentType,
												kOSAGenericScriptingComponentSubtype );
//														set for AppleScript	
		myOsaErr = ::OSASetDefaultScriptingComponent( TRunOSA::scriptComponent, kAppleScriptSubtype );
		if ( noErr != myOsaErr )
			return( myOsaErr );
		}
//														build AEDesc by hand
	::AEDisposeDesc( &f_aedescScript );

	f_aedescScript.descriptorType = typeOSAGenericStorage;
	f_aedescScript.dataHandle = hScript;
//														kill any existing script (prevent mem. leaks)
	if ( kOSANullScript != f_osaidOriginal )
		::OSADispose( TRunOSA::scriptComponent, f_osaidOriginal );
//														now load pre-compiled script
	myOsaErr = ::OSALoad( TRunOSA::scriptComponent, &f_aedescScript, 
					kOSAModeNull, &f_osaidOriginal );
//														dummy-load 'OSADisplay()'
	//	This will fail, but the idea is that once OSADisplay()
	//	is called, any later calls to it are 10x faster!

	::OSADisplay( TRunOSA::scriptComponent, f_osaidOriginal,
				typeChar, kOSAModeDisplayForHumans,
				&aedescDummy );

	::AEDisposeDesc( &aedescDummy );

	return( myOsaErr );
}



OSAError				TRunOSA::runScript ( void )
//
//	Simply execute loaded script without
//	resorting to any AppleEvent tricks
//
{
//														bypass if not ready
	if ( NULL == TRunOSA::scriptComponent )
		return( errOSAInvalidID );
//														kill any existing result (prevent mem. leaks)
	if ( kOSANullScript != f_osaidResult )
		::OSADispose( TRunOSA::scriptComponent, f_osaidResult );

	return( ::OSAExecute( TRunOSA::scriptComponent, f_osaidOriginal, 
						kOSANullScript, kOSAModeNull, &f_osaidResult ) );
}



OSAError				TRunOSA::runScript ( const AEEventClass		aeclassSuite, 
									const AEEventID 		aesuiteKind,
									Str255 				str255Var )
//
//	Execute the script with the 'str255Var' provided
//
{
	AEDesc		aedescVar		= { typeNull, NULL };
	OSAError		myOsaErr		= noErr;

//														idiot filter
	ASSERT( str255Var[0] > 0 );
//														create AEDesc containing variable
	myOsaErr = ::AECreateDesc( typeChar, &str255Var[1], 
						str255Var[0], &aedescVar );
//														now execute script
	if ( noErr == myOsaErr )
		myOsaErr = myRunScript( aeclassSuite, aesuiteKind, &aedescVar );
//														tidy up
	::AEDisposeDesc( &aedescVar );

	return( myOsaErr );
}



OSAError				TRunOSA::myRunScript ( const AEEventClass	aeclassSuite, 
									const AEEventID 		aesuiteKind,
									AEDesc 				* ptraedescVar )
//
//	Execute the script with the 'ptraedescVar' provided.
//	The two 'AEEvent' items are needed to indicate what kind 
//	of AppleEvent is to be sent to the script
//
{
	AppleEvent	aeEvent		= { typeNull, NULL };
	OSAError		myOsaErr		= noErr;

//														bypass if not ready
	if ( TRunOSA::scriptComponent == NULL )
		return( errOSAInvalidID );
//														idiot filters
	ASSERT( aeclassSuite > 0L );
	ASSERT( aesuiteKind > 0L );
//														build AppleEvent targeted on script AEDesc
	myOsaErr = ::AECreateAppleEvent( aeclassSuite,
							aesuiteKind,
							&f_aedescScript,
							kAutoGenerateReturnID,
							kAnyTransactionID,
							&aeEvent );
//														slot any AEDesc into AppleEvent
	if ( NULL != ptraedescVar && noErr == myOsaErr )
		myOsaErr = ::AEPutParamDesc( &aeEvent, keyDirectObject, ptraedescVar );
//														kill any existing result (prevent mem. leaks)
	if ( noErr == myOsaErr && ( kOSANullScript != f_osaidResult ) )
		::OSADispose( TRunOSA::scriptComponent, f_osaidResult );
//														now send AppleEvent to script & execute
	if ( noErr == myOsaErr )
		myOsaErr = ::OSAExecuteEvent( TRunOSA::scriptComponent, 
								&aeEvent, f_osaidOriginal, 
								kOSAModeNull, &f_osaidResult );
//														tidy up
	::AEDisposeDesc( &aeEvent );

	return( myOsaErr );
}



OSAError				TRunOSA::getResultDesc ( AEDesc 	* ptraedescResult,
										long		* ptrlgSize )
//
//	After a script run, extract any result
//	into a AEDesc pointer provided, as well
//	its size as a long
//	Note:
//	After result is extracted, the 'f_osaidResult'
//	script is deleted (to prevent mega memory leaks!)
//
{
	OSAError	myOsaErr = noErr;

//														bypass if no results
	if ( NULL == TRunOSA::scriptComponent && 
		kOSANullScript == f_osaidResult )
		return( errOSAInvalidID );
//														idiot filters
	ASSERT( NULL != ptraedescResult );	
	ASSERT( NULL != ptrlgSize );	
//														extract result
	myOsaErr = ::OSADisplay( TRunOSA::scriptComponent, f_osaidResult,
						typeChar, kOSAModeDisplayForHumans,
						ptraedescResult );
//														extract size of result
	if ( myOsaErr == noErr )
		*ptrlgSize = ::GetHandleSize( ptraedescResult->dataHandle );
	else
		*ptrlgSize = 0L;
//														tidy up
	::OSADispose( TRunOSA::scriptComponent, f_osaidResult );
	f_osaidResult = kOSANullScript;

	return( myOsaErr );
}
